@@ -1,3 +1,7 @@
+"""
+Extra unittests
+"""
+
 import subprocess
 import os
 
@@ -6,6 +10,10 @@ import os
 
 
 def test_binary_c_installed():
+    """
+    Unittest to check if binary_c actually exists
+    """
+
     binary_c_dir = os.getenv("BINARY_C", None)
 
     assert (
@@ -17,6 +25,10 @@ def test_binary_c_installed():
 
 
 def test_binary_c_version():
+    """
+    Unittest to check if binary_c has the correct version
+    """
+
     required_binary_c_versions = ["2.1.7"]
 
     binary_c_dir = os.getenv("BINARY_C", None)
