@@ -5,6 +5,7 @@ Unittests for the custom_logging module
 import unittest
 
 from binarycpython.utils.custom_logging_functions import *
+from binarycpython.utils.functions import Capturing
 
 binary_c_temp_dir = temp_dir()
 
@@ -15,6 +16,11 @@ class test_custom_logging(unittest.TestCase):
     """
 
     def test_autogen_C_logging_code(self):
+        with Capturing() as output:
+            self._test_autogen_C_logging_code()
+        # print("\n".join(output))
+
+    def _test_autogen_C_logging_code(self):
         """
         Tests for the autogeneration of a print statement from a dictionary. and then checking if the output is correct
         """
@@ -43,6 +49,11 @@ class test_custom_logging(unittest.TestCase):
         self.assertEqual(output_3, None, msg="Output should be None")
 
     def test_binary_c_log_code(self):
+        with Capturing() as output:
+            self._test_binary_c_log_code()
+        # print("\n".join(output))
+
+    def _test_binary_c_log_code(self):
         """
         Test to see if passing a print statement to the function results in correct binary_c output
         """
@@ -53,7 +64,8 @@ class test_custom_logging(unittest.TestCase):
 
         input_2 = 'Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));'
         output_2 = binary_c_log_code(input_2, verbose=1)
-        test_value_2 = '#pragma push_macro("MAX")\n#pragma push_macro("MIN")\n#undef MAX\n#undef MIN\n#include "binary_c.h"\n#include "RLOF/RLOF_prototypes.h"\n\n// add visibility __attribute__ ((visibility ("default"))) to it \nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata);\nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata)\n{\n    // struct stardata_t * stardata = (struct stardata_t *)x;\n    Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));;\n}\n\n#undef MAX \n#undef MIN\n#pragma pop_macro("MIN")\n#pragma pop_macro("MAX")    '
+        test_value_2 = '#pragma push_macro("Max")\n#pragma push_macro("Min")\n#undef Max\n#undef Min\n#include "binary_c.h"\n\n// add visibility __attribute__ ((visibility ("default"))) to it \nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata);\nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata)\n{\n    // struct stardata_t * stardata = (struct stardata_t *)x;\n    Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));;\n}\n\n#undef Max \n#undef Min\n#pragma pop_macro("Min")\n#pragma pop_macro("Max")    '
+
         self.assertEqual(
             output_2,
             test_value_2,
@@ -61,11 +73,17 @@ class test_custom_logging(unittest.TestCase):
         )
 
     def test_binary_c_write_log_code(self):
+        with Capturing() as output:
+            self._test_binary_c_write_log_code()
+        # print("\n".join(output))
+
+    def _test_binary_c_write_log_code(self):
         """
         Tests to see if writing the code to a file and reading that out again is the same
         """
 
-        input_1 = '#pragma push_macro("MAX")\n#pragma push_macro("MIN")\n#undef MAX\n#undef MIN\n#include "binary_c.h"\n#include "RLOF/RLOF_prototypes.h"\n\n// add visibility __attribute__ ((visibility ("default"))) to it \nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata);\nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata)\n{\n    // struct stardata_t * stardata = (struct stardata_t *)x;\n    Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));;\n}\n\n#undef MAX \n#undef MIN\n#pragma pop_macro("MIN")\n#pragma pop_macro("MAX")    '
+        input_1 = '#pragma push_macro("Max")\n#pragma push_macro("Min")\n#undef Max\n#undef Min\n#include "binary_c.h"\n\n// add visibility __attribute__ ((visibility ("default"))) to it \nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata);\nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata)\n{\n    // struct stardata_t * stardata = (struct stardata_t *)x;\n    Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));;\n}\n\n#undef Max \n#undef Min\n#pragma pop_macro("Min")\n#pragma pop_macro("Max")    '
+
         binary_c_write_log_code(
             input_1,
             os.path.join(binary_c_temp_dir, "test_binary_c_write_log_code.txt"),
@@ -85,6 +103,11 @@ class test_custom_logging(unittest.TestCase):
         self.assertEqual(repr(input_1), content_file, msg="Contents are not similar")
 
     def test_from_binary_c_config(self):
+        with Capturing() as output:
+            self._test_from_binary_c_config()
+        # print("\n".join(output))
+
+    def _test_from_binary_c_config(self):
         """
         Tests for interfacing with binary_c-config
         """
@@ -106,9 +129,14 @@ class test_custom_logging(unittest.TestCase):
 
         input_2 = "version"
         output_2 = from_binary_c_config(BINARY_C_CONFIG, input_2)
-        self.assertEqual(output_2, "2.1.7", msg="binary_c version doesnt match")
+        self.assertIn(output_2, ["2.1.7", "2.2pre1"], msg="binary_c version doesnt match")
 
     def test_return_compilation_dict(self):
+        with Capturing() as output:
+            self._test_return_compilation_dict()
+        # print("\n".join(output))
+
+    def _test_return_compilation_dict(self):
         """
         Tests to see if the compilation dictionary contains the correct keys
         """
@@ -125,12 +153,17 @@ class test_custom_logging(unittest.TestCase):
         self.assertTrue("inc" in output)
 
     def test_create_and_load_logging_function(self):
+        with Capturing() as output:
+            self._test_create_and_load_logging_function()
+        # print("\n".join(output))
+
+    def _test_create_and_load_logging_function(self):
         """
         Tests checking the output of create_and_load_logging_function. Should return a valid memory int and a correct filename
         """
 
         #
-        input_1 = '#pragma push_macro("MAX")\n#pragma push_macro("MIN")\n#undef MAX\n#undef MIN\n#include "binary_c.h"\n#include "RLOF/RLOF_prototypes.h"\n\n// add visibility __attribute__ ((visibility ("default"))) to it \nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata);\nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata)\n{\n    // struct stardata_t * stardata = (struct stardata_t *)x;\n    Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));;\n}\n\n#undef MAX \n#undef MIN\n#pragma pop_macro("MIN")\n#pragma pop_macro("MAX")    '
+        input_1 = '#pragma push_macro("MAX")\n#pragma push_macro("MIN")\n#undef MAX\n#undef MIN\n#include "binary_c.h"\n\n// add visibility __attribute__ ((visibility ("default"))) to it \nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata);\nvoid binary_c_API_function custom_output_function(struct stardata_t * stardata)\n{\n    // struct stardata_t * stardata = (struct stardata_t *)x;\n    Printf("MY_STELLAR_DATA %g %g %g %g\\n",((double)stardata->model.time),((double)stardata->star[0].mass),((double)stardata->model.probability),((double)stardata->model.dt));;\n}\n\n#undef MAX \n#undef MIN\n#pragma pop_macro("MIN")\n#pragma pop_macro("MAX")    '
         output_1 = create_and_load_logging_function(input_1, verbose=1)
 
         self.assertTrue(isinstance(output_1[0], int), msg="memaddr is not an int")
