@@ -16,6 +16,13 @@ All the options starting with _ should not be changed by the user except when yo
 import os
 
 from binarycpython.utils.custom_logging_functions import temp_dir
+from binarycpython.utils.functions import return_binary_c_version_info
+
+_LOGGER_VERBOSITY_LEVEL = 1
+_CUSTOM_LOGGING_VERBOSITY_LEVEL = 2
+_MS_VERBOSITY_LEVEL = 5
+_MS_VERBOSITY_INTERPOLATOR_LEVEL = 6
+_MS_VERBOSITY_INTERPOLATOR_EXTRA_LEVEL = 7
 
 # Options dict
 grid_options_defaults_dict = {
@@ -23,18 +30,23 @@ grid_options_defaults_dict = {
     # general (or unordered..)
     ##########################
     "amt_cores": 1,  # total amount of cores used to evolve the population
-    "binary": 0,  # FLag on whether the systems are binary systems or single systems.
-    "parse_function": None,  # FUnction to parse the output with.
+    "parse_function": None,  # Function to parse the output with.
+    "multiplicity_fraction_function": 0, # Which multiplicity fraction function to use. 0: None, 1: Arenou 2010, 2: Rhagavan 2010, 3: M&S 2017
     "tmp_dir": temp_dir(),  # Setting the temp dir of the program
     "_main_pid": -1,  # Placeholder for the main process id of the run.
-    "combine_ensemble_with_thread_joining": True,  # Flag on whether to combine everything and return it to the user or if false: write it to data_dir/ensemble_output_{popuation_id}_{thread_id}.json
-    # "output_dir":
+    "combine_ensemble_with_thread_joining": True,  # Flag on whether to combine everything and return it to the user or if false: write it to data_dir/ensemble_output_{population_id}_{thread_id}.json
     "_commandline_input": "",
-    "log_runtime_systems": 0, # whether to log the runtime of the systems (1 file per thread. stored in the tmp_dir)
+    "log_runtime_systems": 0,  # whether to log the runtime of the systems (1 file per thread. stored in the tmp_dir)
+    "_actually_evolve_system": True,  # Whether to actually evolve the systems of just act as if. for testing. used in _process_run_population_grid
+    "max_queue_size": 1000,  # Maximum size of the system call queue.
+    "run_zero_probability_system": True, # Whether to run the zero probability systems
+    "_zero_prob_stars_skipped": 0,
+    "ensemble_factor_in_probability_weighted_mass": False, # Whether to multiply the ensemble results by 1/probability_weighted_mass
+    "do_dry_run": True, # Whether to do a dry run to calculate the total probability for this run
     ##########################
     # Execution log:
     ##########################
-    "verbosity": 0,  # Level of verbosity of the simulation. 0=INFO,
+    "verbosity": 0,  # Level of verbosity of the simulation
     "log_file": os.path.join(
         temp_dir(), "binary_c_python.log"
     ),  # Set to None to not log to file. The directory will be created
@@ -52,19 +64,25 @@ grid_options_defaults_dict = {
     ),  # TODO: make this more robust
     "_binary_c_dir": os.environ["BINARY_C"],
     ##########################
+    # M&S internal settings
+    ##########################
+    "_loaded_ms_data": False, # Holds flag whether the M&S data is loaded into memory
+    "_set_ms_grid": False, # Whether the M&S grid has been loaded
+    "m&s_options": None, # Holds the M&S options.
+    ##########################
     # Custom logging
     ##########################
-    "C_auto_logging": None,  # Should contain a dictionary where the kes are they headers
+    "C_auto_logging": None,  # Should contain a dictionary where the keys are they headers
     # and the values are lists of parameters that should be logged.
-    # This will get parsed by autogen_C_logging_code in custom_loggion_functions.py
+    # This will get parsed by autogen_C_logging_code in custom_logging_functions.py
     "C_logging_code": None,  # Should contain a string which holds the logging code.
     "custom_logging_func_memaddr": -1,  # Contains the custom_logging functions memory address
     "_custom_logging_shared_library_file": None,  # file containing the .so file
     ##########################
     # Store pre-loading:
     ##########################
-    "_store_memaddr": -1,  # Contains the store object memory adress, useful for preloading.
-    # defaults to -1 and isnt used if thats the default then.
+    "_store_memaddr": -1,  # Contains the store object memory address, useful for pre loading.
+    # defaults to -1 and isn't used if that's the default then.
     ##########################
     # Log args: logging of arguments
     ##########################
@@ -77,7 +95,7 @@ grid_options_defaults_dict = {
     "evolution_type": "grid",  # Flag for type of population evolution
     "_evolution_type_options": [
         "grid",
-    ],  # available choices for type of population evolution. # TODO: fill later with monte carlo, sourcefile
+    ],  # available choices for type of population evolution. # TODO: fill later with Monte Carlo, source file
     "_system_generator": None,  # value that holds the function that generates the system
     # (result of building the grid script)
     "source_file_filename": None,  # filename for the source
@@ -95,6 +113,8 @@ grid_options_defaults_dict = {
     "failed_systems_threshold": 20,  # Maximum failed systems per process allowed to fail before the process stops logging the failing systems.
     "_failed_systems_error_codes": [],  # List to store the unique error codes
     "_population_id": 0,  # Random id of this grid/population run, Unique code for the population. Should be set only once by the controller process.
+    "_total_mass_run": 0,  # To count the total mass that thread/process has ran
+    "_total_probability_weighted_mass_run": 0,  # To count the total mass * probability for each system that thread/process has ran
     "modulo": 1,  # run modulo n of the grid. #TODO: fix this
     ## Grid type evolution
     "_grid_variables": {},  # grid variables
@@ -427,14 +447,14 @@ grid_options_descriptions = {
     "_binary_c_shared_library": "Full path to the libbinary_c file. This options is not used in the population object",
     "verbosity": "Verbosity of the population code. Default is 0, by which only errors will be printed. Higher values will show more output, which is good for debugging.",
     "binary": "Set this to 1 if the population contains binaries. Input: int",  # TODO: write what effect this has.
-    "amt_cores": "The amount of cores that the population grid will use. The multiprocessing is useful but make sure to figure out how many logical cores the machine has. The core is multiprocessed, not multithreaded, and will gain no extra speed when amt_cores exceeds the amount of logical cores. Input: int",
+    "amt_cores": "The amount of cores that the population grid will use. The multiprocessing is useful but make sure to figure out how many logical cores the machine has. The core is multi processed, not multi threaded, and will gain no extra speed when amt_cores exceeds the amount of logical cores. Input: int",
     "_start_time_evolution": "Variable storing the start timestamp of the population evolution. Set by the object itself.",  # TODO: make sure this is logged to a file
     "_end_time_evolution": "Variable storing the end timestamp of the population evolution. Set by the object itself",  # TODO: make sure this is logged to a file
     "_total_starcount": "Variable storing the total amount of systems in the generator. Used and set by the population object.",
     "_custom_logging_shared_library_file": "filename for the custom_logging shared library. Used and set by the population object",
-    "_errors_found": "Variable storing a boolean flag whether errors by binary_c are encountered.",
-    "_errors_exceeded": "Variable storing a boolean flag whether the amount of errors was higher than the set threshold (failed_systems_threshold). If True, then the commandline arguments of the failing systems will not be stored in the failed_system_log files.",
-    "source_file_filename": "Variable containing the source file containing lines of binary_c commandline calls. These all have to start with binary_c.",  # TODO: Expand
+    "_errors_found": "Variable storing a Boolean flag whether errors by binary_c are encountered.",
+    "_errors_exceeded": "Variable storing a Boolean flag whether the amount of errors was higher than the set threshold (failed_systems_threshold). If True, then the command line arguments of the failing systems will not be stored in the failed_system_log files.",
+    "source_file_filename": "Variable containing the source file containing lines of binary_c command line calls. These all have to start with binary_c.",  # TODO: Expand
     "C_auto_logging": "Dictionary containing parameters to be logged by binary_c. The structure of this dictionary is as follows: the key is used as the headline which the user can then catch. The value at that key is a list of binary_c system parameters (like star[0].mass)",
     "C_logging_code": "Variable to store the exact code that is used for the custom_logging. In this way the user can do more complex logging, as well as putting these logging strings in files.",
     "_failed_count": "Variable storing the amount of failed systems.",
@@ -444,27 +464,206 @@ grid_options_descriptions = {
     "_grid_variables": "Dictionary storing the grid_variables. These contain properties which are accessed by the _generate_grid_code function",
     "_population_id": "Variable storing a unique 32-char hex string.",
     "_commandline_input": "String containing the arguments passed to the population object via the command line. Set and used by the population object.",
-    "_system_generator": "Function object that contains the system generator function. This can be from a grid, or a source file, or a montecarlo grid.",
+    "_system_generator": "Function object that contains the system generator function. This can be from a grid, or a source file, or a Monte Carlo grid.",
     "gridcode_filename": "Filename for the grid code. Set and used by the population object. TODO: allow the user to provide their own function, rather than only a generated function.",
     "log_args": "Boolean to log the arguments. Unused ",  # TODO: fix the functionality for this and describe it properly
     "log_args_dir": "Directory to log the arguments to. Unused",  # TODO: fix the functionality for this and describe it properly
     "log_file": "Log file for the population object. Unused",  # TODO: fix the functionality for this and describe it properly
-    "custom_logging_func_memaddr": "Memory adress where the custom_logging_function is stored. Input: int",
+    "custom_logging_func_memaddr": "Memory address where the custom_logging_function is stored. Input: int",
     "_count": "Counter tracking which system the generator is on.",
     "_probtot": "Total probability of the population.",  # TODO: check whether this is used properly throughout
     "_main_pid": "Main process ID of the master process. Used and set by the population object.",
-    "_store_memaddr": "Memory adress of the store object for binary_c.",
-    "failed_systems_threshold": "Variable storing the maximum amount of systems that are allowed to fail before logging their commandline arguments to failed_systems log files",
+    "_store_memaddr": "Memory address of the store object for binary_c.",
+    "failed_systems_threshold": "Variable storing the maximum amount of systems that are allowed to fail before logging their command line arguments to failed_systems log files",
     "parse_function": "Function that the user can provide to handle the output the binary_c. This function has to take the arguments (self, output). Its best not to return anything in this function, and just store stuff in the grid_options['results'] dictionary, or just output results to a file",
     "condor": "Int flag whether to use a condor type population evolution. Not implemented yet.",  # TODO: describe this in more detail
-    "slurm": "Int flag whether to use a slurm type population evolution.",  # TODO: describe this in more detail
-    "weight": "Weight factor for each system. The calculated probability is mulitplied by this. If the user wants each system to be repeated several times, then this variable should not be changed, rather change the _repeat variable instead, as that handles the reduction in probability per system. This is useful for systems that have a process with some random element in it.",  # TODO: add more info here, regarding the evolution splitting.
+    "slurm": "Int flag whether to use a Slurm type population evolution.",  # TODO: describe this in more detail
+    "weight": "Weight factor for each system. The calculated probability is multiplied by this. If the user wants each system to be repeated several times, then this variable should not be changed, rather change the _repeat variable instead, as that handles the reduction in probability per system. This is useful for systems that have a process with some random element in it.",  # TODO: add more info here, regarding the evolution splitting.
     "repeat": "Factor of how many times a system should be repeated. Consider the evolution splitting binary_c argument for supernovae kick repeating.",  # TODO: make sure this is used.
     "evolution_type": "Variable containing the type of evolution used of the grid. Multiprocessing or linear processing",
-    "combine_ensemble_with_thread_joining": "BOolean flag on whether to combine everything and return it to the user or if false: write it to data_dir/ensemble_output_{popuation_id}_{thread_id}.json",
-    "log_runtime_systems": "Whether to log the runtime of the systems . Each systems run by the thread is logged to a file and is stored in the tmp_dir. (1 file per thread). Don't use this if you are planning to run alot of systems. This is mostly for debugging and finding systems that take long to run. Integer, default = 0. if value is 1 then the systems are logged",
+    "combine_ensemble_with_thread_joining": "Boolean flag on whether to combine everything and return it to the user or if false: write it to data_dir/ensemble_output_{population_id}_{thread_id}.json",
+    "log_runtime_systems": "Whether to log the runtime of the systems . Each systems run by the thread is logged to a file and is stored in the tmp_dir. (1 file per thread). Don't use this if you are planning to run a lot of systems. This is mostly for debugging and finding systems that take long to run. Integer, default = 0. if value is 1 then the systems are logged",
+    "_total_mass_run": "To count the total mass that thread/process has ran",
+    "_total_probability_weighted_mass_run": "To count the total mass * probability for each system that thread/process has ran",
+    "_actually_evolve_system": "Whether to actually evolve the systems of just act as if. for testing. used in _process_run_population_grid",
+    "max_queue_size": "Maximum size of the queue that is used to feed the processes. Don't make this too big! Default: 1000. Input: int",
+    "_set_ms_grid": "Internal flag whether the M&S grid has been loaded",
+    "run_zero_probability_system": "Whether to run the zero probability systems. Default: True. Input: Boolean",
+    "_zero_prob_stars_skipped": "Internal counter to track how many systems are skipped because they have 0 probability",
+    "ensemble_factor_in_probability_weighted_mass": "Flag to multiply all the ensemble results with 1/probability_weighted_mass",
+    "multiplicity_fraction_function": "Which multiplicity fraction function to use. 0: None, 1: Arenou 2010, 2: Rhagavan 2010, 3: M&S 2017",
+    "m&s_options": "Internal variable that holds the M&S options. Don't write to this your self",
+    "_loaded_ms_data": "Internal variable storing whether the M&S data has been loaded into memory",
+    "do_dry_run": "Whether to do a dry run to calculate the total probability for this run",
 }
 
+###
+#
+
+# Default options for the Moe & di Stefano grid
+moe_di_stefano_default_options = {
+    "resolutions": {
+        "M": [
+            20,  # M1
+            20,  # M2 (i.e. q)
+            0,  # M3 currently unused
+            0,  # M4 currently unused
+        ],
+        "logP": [
+            20,  # P2 (binary period)
+            0,  # P3 (triple period) currently unused
+            0,  # P4 (quadruple period) currently unused
+        ],
+        "ecc": [
+            10,  # e (binary eccentricity)
+            0,  # e2 (triple eccentricity) currently unused
+            0,  # e3 (quadruple eccentricity) currently unused
+        ],
+    },
+    "ranges": {
+        # stellar masses (Msun)
+        "M": [
+            0.08,  # 0.08 is a tad bit above the minimum mass. Don't sample at 0.07, otherwise the first row of q values will have a phasevol of 0. Anything higher is fine.
+            80.0,  # (rather arbitrary) upper mass cutoff
+        ],
+        "q": [
+            None,  # artificial qmin : set to None to use default
+            None,  # artificial qmax : set to None to use default
+        ],
+        "logP": [0.0, 8.0],  # 0 = log10(1 day)  # 8 = log10(10^8 days)
+        "ecc": [0.0, 0.99],
+    },
+    # minimum stellar mass
+    "Mmin": float(return_binary_c_version_info(parsed=True)['macros']['BINARY_C_MINIMUM_STELLAR_MASS']), # We take the value that binary_c has set as the default 
+
+    # multiplicity model (as a function of log10M1)
+    #
+    # You can use 'Poisson' which uses the system multiplicity
+    # given by Moe and maps this to single/binary/triple/quad
+    # fractions.
+    #
+    # Alternatively, 'data' takes the fractions directly
+    # from the data, but then triples and quadruples are
+    # combined (and there are NO quadruples).
+    "multiplicity_model": "Poisson",
+
+    # multiplicity modulator:
+    # [single, binary, triple, quadruple]
+    #
+    # e.g. [1,0,0,0] for single stars only
+    #      [0,1,0,0] for binary stars only
+    #
+    # defaults to [1,1,0,0] i.e. all types
+    #
+    "multiplicity_modulator": [
+        1,  # single
+        1,  # binary
+        0,  # triple
+        0,  # quadruple
+    ],
+    # given a mix of multiplicities, you can either (noting that
+    # here (S,B,T,Q) = appropriate modulator * model(S,B,T,Q) )
+    #
+    # 'norm'  : normalise so the whole population is 1.0
+    #           after implementing the appropriate fractions
+    #           S/(S+B+T+Q), B/(S+B+T+Q), T/(S+B+T+Q), Q/(S+B+T+Q)
+    #
+    # 'raw'   : stick to what is predicted, i.e.
+    #           S/(S+B+T+Q), B/(S+B+T+Q), T/(S+B+T+Q), Q/(S+B+T+Q)
+    #           without normalisation
+    #           (in which case the total probability < 1.0 unless
+    #            all you use single, binary, triple and quadruple)
+    #
+    # 'merge' : e.g. if you only have single and binary,
+    #           add the triples and quadruples to the binaries, so
+    #           binaries represent all multiple systems
+    #           ...
+    #           *** this is canonical binary population synthesis ***
+    #
+    #           Note: if multiplicity_modulator == [1,1,1,1] this
+    #                 option does nothing (equivalent to 'raw').
+    #
+    #
+    # note: if you only set one multiplicity_modulator
+    # to 1, and all the others to 0, then normalising
+    # will mean that you effectively have the same number
+    # of stars as single, binary, triple or quad (whichever
+    # is non-zero) i.e. the multiplicity fraction is ignored.
+    # This is probably not useful except for
+    # testing purposes or comparing to old grids.
+    "normalize_multiplicities": "merge",
+    # q extrapolation (below 0.15 and above 0.9) method. We can choose from ['flat', 'linear', 'plaw2', 'nolowq']
+    "q_low_extrapolation_method": "linear",
+    "q_high_extrapolation_method": "linear",
+}
+
+moe_di_stefano_default_options_description = {
+    "resolutions": "",
+    "ranges": "",
+    "Mmin": "Minimum stellar mass",
+    "multiplicity_model": """
+multiplicity model (as a function of log10M1)
+
+You can use 'Poisson' which uses the system multiplicity
+given by Moe and maps this to single/binary/triple/quad
+fractions.
+
+Alternatively, 'data' takes the fractions directly
+from the data, but then triples and quadruples are
+combined (and there are NO quadruples).
+""",
+    "multiplicity_modulator": """
+[single, binary, triple, quadruple]
+
+e.g. [1,0,0,0] for single stars only
+     [0,1,0,0] for binary stars only
+
+defaults to [1,1,0,0] i.e. singles and binaries
+""",
+    "normalize_multiplicities": """
+'norm': normalise so the whole population is 1.0
+        after implementing the appropriate fractions
+        S/(S+B+T+Q), B/(S+B+T+Q), T/(S+B+T+Q), Q/(S+B+T+Q)
+        given a mix of multiplicities, you can either (noting that
+        here (S,B,T,Q) = appropriate modulator * model(S,B,T,Q) )
+        note: if you only set one multiplicity_modulator
+        to 1, and all the others to 0, then normalising
+        will mean that you effectively have the same number
+        of stars as single, binary, triple or quad (whichever
+        is non-zero) i.e. the multiplicity fraction is ignored.
+        This is probably not useful except for
+        testing purposes or comparing to old grids.
+
+'raw'   : stick to what is predicted, i.e.
+          S/(S+B+T+Q), B/(S+B+T+Q), T/(S+B+T+Q), Q/(S+B+T+Q)
+          without normalisation
+          (in which case the total probability < 1.0 unless
+          all you use single, binary, triple and quadruple)
+
+'merge' : e.g. if you only have single and binary,
+          add the triples and quadruples to the binaries, so
+          binaries represent all multiple systems
+          ...
+          *** this is canonical binary population synthesis ***
+
+          It only takes the maximum multiplicity into account,
+          i.e. it doesn't multiply the resulting array by the multiplicity modulator again.
+          This prevents the resulting array to always be 1 if only 1 multiplicity modulator element is nonzero
+
+          Note: if multiplicity_modulator == [1,1,1,1]. this option does nothing (equivalent to 'raw').
+""",
+    "q_low_extrapolation_method": """
+q extrapolation (below 0.15) method
+    none
+    flat
+    linear2
+    plaw2
+    nolowq
+""",
+    "q_high_extrapolation_method": "Same as q_low_extrapolation_method",
+}
+
+
 #################################
 # Grid options functions
 
@@ -477,7 +676,7 @@ def grid_options_help(option: str) -> dict:
         option: which option you want to have the description of
 
     returns:
-        dict containg the option, the description if its there, otherwise empty string. And if the key doesnt exist, the dict is empty
+        dict containing the option, the description if its there, otherwise empty string. And if the key doesnt exist, the dict is empty
     """
 
     option_keys = grid_options_defaults_dict.keys()
@@ -540,7 +739,7 @@ def write_grid_options_to_rst_file(output_file: str) -> None:
     Function that writes the descriptions of the grid options to a rst file
 
     Tasks:
-        TODO: seperate things into private and public options
+        TODO: separate things into private and public options
 
     Args:
         output_file: target file where the grid options descriptions are written to
@@ -578,41 +777,49 @@ def write_grid_options_to_rst_file(output_file: str) -> None:
         print("\n", file=f)
 
         # Start public options part
-        print("Public options", file=f)
-        print("{}".format("-" * len("Public options")), file=f)
-        print("The following options are meant to be changed by the user.", file=f)
-        print("\n", file=f)
-
-        for public_option in sorted(public_options):
-            if public_option in descriptions:
-                print(
-                    "| **{}**: {}".format(public_option, descriptions[public_option]),
-                    file=f,
-                )
-            else:
-                print(
-                    "| **{}**: No description available yet".format(public_option),
-                    file=f,
-                )
-            print("", file=f)
-
-        # Start private options part
-        print("Private options", file=f)
-        print("{}".format("-" * len("Private options")), file=f)
-        print(
-            "The following options are not meant to be changed by the user, as these options are used and set internally by the object itself. The description still is provided, but just for documentation purposes.",
-            file=f,
+        print_option_descriptions(
+            f,
+            public_options,
+            descriptions,
+            "Public options",
+            "The following options are meant to be changed by the user.",
         )
 
-        for private_option in sorted(private_options):
-            if private_option in descriptions:
-                print(
-                    "| **{}**: {}".format(private_option, descriptions[private_option]),
-                    file=f,
-                )
-            else:
-                print(
-                    "| **{}**: No description available yet".format(private_option),
-                    file=f,
-                )
-            print("", file=f)
+        # Moe & di Stefano options:
+        print_option_descriptions(
+            f,
+            moe_di_stefano_default_options,
+            moe_di_stefano_default_options_description,
+            "Moe & di Stefano sampler options",
+            "The following options are meant to be changed by the user.",
+        )
+
+        # Start private options part
+        print_option_descriptions(
+            f,
+            private_options,
+            descriptions,
+            "Private options",
+            "The following options are not meant to be changed by the user, as these options are used and set internally by the object itself. The description still is provided, but just for documentation purposes.",
+        )
+
+
+def print_option_descriptions(filehandle, options, descriptions, title, extra_text):
+    # Start public options part
+    print("{}".format(title), file=filehandle)
+    print("{}".format("-" * len("{}".format(title))), file=filehandle)
+    print("{}".format(extra_text), file=filehandle)
+    print("\n", file=filehandle)
+
+    for option in sorted(options):
+        if option in descriptions:
+            print(
+                "| **{}**: {}".format(option, descriptions[option].replace("\n", "\n\t")),
+                file=filehandle,
+            )
+        else:
+            print(
+                "| **{}**: No description available yet".format(option),
+                file=filehandle,
+            )
+        print("", file=filehandle)
