@@ -9,18 +9,19 @@ Structure of file:
     plotting functions
     master function: plot_system
 
-All the loose components here can ofcourse be used in other routines if you want.
+All the loose components here can of course be used in other routines if you want.
 
-There is no preloaded matplotlib rc, you should do that yourself
+There is no pre loaded matplotlib rc, you should do that yourself
 
-These plotting routines are designed for binary systems, and admittingly they are here mostly for 
-inspirational purposes, since one would problably want to customize the plots. 
+These plotting routines are designed for binary systems, and admittedly they are here mostly for 
+inspirational purposes, since one would probably want to customise the plots. 
 Regardless, having some plotting routines in here seemed like a nice idea
 
 Tasks
     TODO: This module is not finished yet.
     TODO: Make modules for single system
     TODO: Put all the plotting functions in here
+
 """
 
 import math
@@ -39,7 +40,7 @@ from binarycpython.utils.custom_logging_functions import binary_c_log_code
 # These are kept to the minimum necessary for each plotting routine.
 
 CUSTOM_LOGGING_STRING_MASSES = """
-Printf("MASS_PLOTTING %30.12e %g %g %g %g\\n",
+Printf("MASS_PLOTTING %30.12e %g %g %g %g %g %g\\n",
     //
     stardata->model.time, // 1
     
@@ -48,8 +49,10 @@ Printf("MASS_PLOTTING %30.12e %g %g %g %g\\n",
     stardata->common.zero_age.mass[1], //
 
     stardata->star[0].mass,
-    stardata->star[1].mass
-    );
+    stardata->star[1].mass,
+    stardata->star[0].core_mass[ID_core(stardata->star[0].stellar_type)],
+    stardata->star[1].core_mass[ID_core(stardata->star[1].stellar_type)]
+);
 """
 
 CUSTOM_LOGGING_STRING_ORBIT = """
@@ -341,7 +344,7 @@ def plot_masses(df, show_stellar_types: bool = False, show_plot: bool = True):
     )
 
     # Mass 1
-    fig.axes[0].plot(df["time"], df["mass_1"], label="star 1")
+    fig.axes[0].plot(df["time"], df["mass_1"], label="star 1", color='red')
     fig.axes[0].axhline(
         df["pms_mass_1"].values.tolist()[0],
         color="red",
@@ -350,17 +353,33 @@ def plot_masses(df, show_stellar_types: bool = False, show_plot: bool = True):
         label="Initial mass 1",
         alpha=0.5,
     )
+    fig.axes[0].plot(
+        df["time"], df["core_mass_1"],
+        color="red",
+        linestyle="-.",
+        linewidth=2,
+        label="Core mass 1",
+        alpha=0.5,
+    )
 
     # mass 2
     fig.axes[0].plot(df["time"], df["mass_2"], color="orange", label="star 2")
     fig.axes[0].axhline(
         df["pms_mass_2"].values.tolist()[0],
-        color="red",
+        color="orange",
         linestyle="--",
         linewidth=2,
         label="Initial mass 2",
         alpha=0.5,
     )
+    fig.axes[0].plot(
+        df["time"], df["core_mass_2"],
+        color="orange",
+        linestyle="-.",
+        linewidth=2,
+        label="Core mass 2",
+        alpha=0.5,
+    )
 
     # Make up
     fig.axes[0].set_title("Stellar mass evolution")
@@ -484,6 +503,8 @@ def parse_function_masses(output: str):
         "pms_mass_2",
         "mass_1",
         "mass_2",
+        "core_mass_1",
+        "core_mass_2",
     ]
 
     # Go over the output.
@@ -511,7 +532,7 @@ def plot_system(plot_type, **kwargs):
         - a preset custom logging for a specific plotting routine is loaded, depending on the choice of plot_type
         - This is used for the run_system call
         - The output of this run_system is loaded into a dataframe by
-        parsing it with a corresponding parsing function
+            parsing it with a corresponding parsing function
         - The dataframe is passed to the plotting routine
         - plot is shown or returned.
 
@@ -526,16 +547,14 @@ def plot_system(plot_type, **kwargs):
         - TODO: make the plotting specific keywords available via the inspect stuff
 
     All keywords are considered kwargs, except for plot_type
+
     Args:
         plot_type: string input should be one of ['mass_evolution', 'orbit_evolution', 'hr_diagram'].
             Input will be matched against this, and then go through a dictionary to pick the correct plotting function.
-
         show_plot: boolean whether to show the plot. If False it returns the figure object
             (makes so that you can customize it)
-
         show_stellar_types: whether to plot the stellar type evolution on a second pane.
             This is not included in all the plotting routines.
-
         Other input: other kwargs that are passed to run_system
             (inspect the docstring of run_system for more info)
 
