@@ -21,7 +21,7 @@ def const(
         steps: amount of segments between min_bound and max_bound
 
     Returns:
-        np.linspace(min_bound, max_bound, steps)
+        np.linspace(min_bound, max_bound, steps+1)
     """
 
-    return np.linspace(min_bound, max_bound, steps)
+    return np.linspace(min_bound, max_bound, steps+1)
