@@ -1,5 +1,7 @@
 """
-Module containing stellar type dicts
+Module containing two stellar type dicts:
+    - STELLAR_TYPE_DICT: dictionary with long names
+    - STELLAR_TYPE_DICT_SHORT: dictionary with short names/abbreviations
 """
 
 STELLAR_TYPE_DICT = {
